package com.cmb.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.cmb.service.HclcCmbPolyService;
import com.cmb.util.*;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.hand.hap.core.IRequest;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;
import java.util.stream.Collectors;

@Service
public class HclcCmbPolyServiceImpl implements HclcCmbPolyService {
    private static final String url = "https://api.cmburl.cn:8065/polypay/v1.0/mchorders/qrcodeapply";  //UAT

    //需要补充域名加端口==================================================================
    private static final String notifyUrl = "/cmb/poly/order/notify";
    //uat环境商户国密私钥
    public static final String privateKey = "D5F2AFA24E6BA9071B54A8C9AD735F9A1DE9C4657FA386C09B592694BC118B38";
    //uat环境招行国密公钥
    public static final String publicKey = "MFkwEwYHKoZIzj0CAQYIKoEcz1UBgi0DQgAE6Q+fktsnY9OFP+LpSR5Udbxf5zHCFO0PmOKlFNTxDIGl8jsPbbB/9ET23NV+acSz4FEkzD74sW2iiNVHRLiKHg==";

    public static final String appId = "8ab74856-8772-45c9-96db-54cb30ab9f74";
    public static final String appSecret = "5b96f20a-011f-4254-8be8-9a5ceb2f317f";

    @Value("${cmb.version}")
    private String version;
    @Value("${cmb.encoding}")
    private String encoding;
    @Value("${cmb.signMethod}")
    private String signMethod;

    /*币种，非必传，默认156人民币*/
    @Value("${cmb.currencyCode}")
    private String currencyCode;

    @Override
    public JSONObject getQrcode(IRequest iRequest, JSONObject params) {
        /*
        版本号	    version	        String(5)	    是	固定为0.0.1
        编码方式	    encoding	    String(20)	    是	固定为UTF-8
        签名	        sign	        String(1024)	是
        签名方法	    signMethod	    String(2)	    是	02：SM2
        商户号	    merId	        String(32)	    是	商户在招行完成商户进件后分配的招行商户号
        商户订单号	orderId	        String(32)	    是	商户端生成，要求此订单号在整个商户下唯一
        收银员	    userId	        String(32)	    是	招行系统生成的收银员，每个收银员均会对应到一个门店。若商户无门店、收银员概念则上送默认的虚拟收银员即可。
        终端号	    termId	        String(8)	    否	原则是可以通过交易上送的终端编号准确定位商户每一个门店内每一台收银设备，建议按“门店编号+收银机编号”或“设备编号”组成8位终端编号在交易中上送
        支付有效时间	payValidTime	String(10)	    否	二维码的有效时长，单位为秒，若商户上送，则以商户上送值为准，否则使用默认有效时长1200秒
        交易通知地址	notifyUrl	    String(256)	    是	支付结果将送到本通知
        交易金额	    txnAmt	        String(13)	    是	单位为分
        交易币种	    currencyCode	String(3)	    否	默认156，目前只支持人民币（156）
        商品描述	    body	        String(128)	    否	用于展示支付宝账单详情中的“商品说明”和微信账单详情中的“商品”字段，若不上送则默认显示商户简称。
        商户保留域	mchReserved	    String(128)     否	用于传输商户自定义数据，在支付结果查询和支付结果通知时原样返回（不上送则不返回此字段）
        交易场景	    tradeScene	    String(13)	    是	OFFLINE：线下
                                                        ONLINE：线上
                                                        INSURANCE：保险
                                                        CHARITY：公益
                                                        PAYMENT: 缴费
                                                        SCHOOL：校园
        商户门店号	mchStoreId	    String(32)	    否	商户门店号
        终端信息	    terminalInfo	String(6000)	否	商户侧受理终端信息，字段说明详见终端信息terminalInfo。
        */



        JSONObject result = new JSONObject();

        Map<String, String> requestPublicParams = JSONObject.parseObject(params.toJSONString(),Map.class);
        // 组装requestBody并加签
        String signResult= signMethod(requestPublicParams);
        try {
            ObjectMapper mapper = new ObjectMapper();
            Map<String,String> signResultMap = mapper.readValue(signResult, Map.class);
            long currentTimeMills = System.currentTimeMillis() / 1000;

            // 组apiSign加密Map
            Map<String,String> apiSign = new TreeMap<>();
            apiSign.put("appid", appId);
            apiSign.put("secret", appSecret);
            apiSign.put("sign", signResultMap.get("sign"));
            apiSign.put("timestamp", "" + currentTimeMills);

            // MD5加密
            String MD5Content = SignatureUtil.getSignContent(apiSign);
            String apiSignString = MD5Utils.getMD5Content(MD5Content).toLowerCase();

            // 组request头部Map
            Map<String, String> apiHeader = new HashMap<>();
            apiHeader.put("appid", appId);
            apiHeader.put("timestamp", "" + currentTimeMills);
            apiHeader.put("apisign", apiSignString);

            // 发送HTTP post请求
            Map<String,String> response = Utils.postForEntity(url, signResult, apiHeader);
/*
            版本号	    version	String(5)	    是	固定为0.0.1
            编码方式	    encoding	String(20)	是	固定为UTF-8
            签名	sign	String(1024)	        是
            签名方法    	signMethod	String(2)	是	02：SM2
            返回码	    returnCode	String(7)	是	SUCCESS/FAIL,此字段是通信标识，非交易标识，交易是否成功需要查看respCode来判断。SUCCESS表示商户上送的报文符合规范，FAIL表示报文内的字段不符合规范，包括长度超长、非法字符、签名错误等
                    以下字段仅在returnCode为SUCCESS时返回
            商户号	    merId	String(32)	    是	请求报文中的商户号
            商户订单号   	orderId	String(32)	    是	请求报文中的商户订单号
            响应码 	    respCode	String(10)	是	业务错误码，成功为SUCCESS，失败为FAIL
                    以下字段仅在returnCode和respCode为SUCCESS时返回
            二维码	    qrCode	String(300)	    是
            平台订单号	cmbOrderId	String(32)	是	招行生成的订单号
            订单发送时间	txnTime	String(14)	    是	订单生成时间，格式为yyyyMMddHHmmss
                    以下字段仅在returnCode或respCode为FAIL时返回
            错误码	    errCode	String(32)	    是	请求处理失败的错误码信息
            应答信息	    respMsg	String(256)	    是	请求处理失败的详细描述信息
*/

            System.out.println(mapper.writeValueAsString(response));

            // 返回结果验签,验证数据的正确性（安全性）
            Boolean checkResult1 = checkSign(mapper.writeValueAsString(response));
            if(!checkResult1){
                //数据验签失败的处理，重发或者直接告诉前端发送失败
                result.put("errCode",response.get("errCode"));
            }
            //获取响应结果内容
            String success = response.get("returnCode");
            System.out.println("返回结果：" + success);
            result.put("qrcode",response.get("qrcode"));
            result.put("respMsg",response.get("respMsg"));
            //业务数据的封装。。。
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            result.put("errCode","服务器问题");
            return result;
        }
    }



    private String signMethod(Map paramsMap){
        Map<String, String> requestPublicParams = new TreeMap<>();
        String requestStr = null;
        try {
            //公共请求参数
            requestPublicParams.put("version", version);    //版本号，固定为0.0.1(必传字段)
            requestPublicParams.put("encoding", encoding);   //编码方式，固定为UTF-8(必传)
            requestPublicParams.put("signMethod", signMethod);    //签名方法，固定为02，表示签名方式为国密(必传)
            requestPublicParams.put("notifyUrl", notifyUrl);
//            //业务要素
//            Map<String, String> requestTransactionParams = new HashMap<>();
//            requestTransactionParams.put("body", "聚合支付测试");   //商户号(必传)
//            requestTransactionParams.put("currencyCode", "156");    //交易币种，默认156，目前只支持人民币（156）
//            requestTransactionParams.put("merId", "3089991074200KV");   //商户号(必传)
//            requestTransactionParams.put("notifyUrl", notifyUrl);  //交易通知地址(必传)
//            requestTransactionParams.put("orderId", "" + System.currentTimeMillis()); //商户订单号(必传)
//            requestTransactionParams.put("payValidTime", "1200"); //支付有效时间
//            requestTransactionParams.put("termId", "00774411");  //终端号
//            requestTransactionParams.put("txnAmt", "1405");  //交易金额,单位为分(必传)
//            requestTransactionParams.put("userId", "N003363806");   //收银员
//            requestTransactionParams.put("tradeScene", "OFFLINE");   //交易场景
            ObjectMapper mapper = new ObjectMapper();
            requestPublicParams.put("biz_content", mapper.writeValueAsString(paramsMap));

            System.out.println("加签前的报文内容：" + mapper.writeValueAsString(requestPublicParams));

            //对待加签内容进行排序拼接
            String signContent= SignatureUtil.getSignContent(requestPublicParams);
            //加签
            requestPublicParams.put("sign", SM2Util.sm2Sign(signContent, privateKey));

            requestStr = mapper.writeValueAsString(requestPublicParams);

            System.out.println("加签后的报文内容：" + requestStr);
            return requestStr;

        }catch (Exception e){
            System.out.println("加签发生异常！");
            e.printStackTrace();
            return requestStr;
        }
    }

    private Boolean checkSign(String string){
        System.out.println("要验签的报文内容：" + string);
        try {
            //验签
            ObjectMapper objectMapper = new ObjectMapper();
            Map<String, String> responseBodyMap = objectMapper.readValue(string, Map.class);
            String sign = responseBodyMap.remove("sign");
            String contentStr = SignatureUtil.getSignContent(responseBodyMap);
            boolean result = SM2Util.sm2Check(contentStr,sign, publicKey);

            if (result) {
                System.out.println("报文验签成功!");
            } else {
                System.out.println("报文验签失败!");
            }
            return result;
        }catch (Exception e){
            System.out.println("验签发生异常！");
            e.printStackTrace();
            return false;
        }
    }

    @Override
    public JSONObject queryOrderStatus(IRequest iRequest, JSONObject params) {
        JSONObject result = new JSONObject();
        final String url = "https://api.cmburl.cn:8065/polypay/v1.0/mchorders/orderquery" ;
        //初始化请求报文


        //实体类为商户传过来的一些参数，会封装作为biz_content部分
        Pojo pojo = new Pojo();
        String requestString = GenerateUtil.initRequestParam(pojo);

            try {
                ObjectMapper objectMapper = new ObjectMapper();
                Map<String,String> requestMap = objectMapper.readValue(requestString,Map.class);

                long currentTimeMills = System.currentTimeMillis() / 1000;

                //对请求报文进行加密
                Map<String,String> apiEncMap = new HashMap<>();
                apiEncMap.put("appid",appId);
                apiEncMap.put("secret",appSecret);
                apiEncMap.put("sign",requestMap.get("sign"));
                apiEncMap.put("timestamp",""+currentTimeMills);

                //MD5加密
                String MD5Content = SignatureUtil.getSignContent(apiEncMap);
                String apiEncString = MD5Utils.getMD5Content(MD5Content).toLowerCase();

                //组请求报文头
                Map<String,String> requstHeader = new HashMap<>();
                requstHeader.put("appid",appId);
                requstHeader.put("timestamp",""+currentTimeMills);
                requstHeader.put("apisign",apiEncString);

                //发送http请求
                Map<String,String> responseMap = Utils.postForEntity(url,requestString,requstHeader);

                //处理响应报文
                GenerateUtil.handleResponse(responseMap);


            }catch (Exception ex) {
                ex.printStackTrace();
            }


        return result;
    }

    @Override
    public Map<String, String> orderNotify(IRequest iRequest, JSONObject params) {
        /*
        版本号	version	String(5)	是	固定为0.0.1
        编码方式	encoding	String(20)	是	固定为UTF-8
        签名	sign	String(1024)	是
        签名方法	signMethod	String(2)	是	02：SM2
        商户号	merId	String(32)	是	商户在招行完成商户进件后分配的招行商户号
        商户订单号	orderId	String(32)	是	支付交易的商户订单号
        平台订单号	cmbOrderId	String(32)	是	招行生成的订单号
        收银员	userId	String(32)	否	支付交易的收银员信息，若有，则返回
        交易金额	txnAmt	String(13)	是	单位为分
        优惠金额	dscAmt	String(13)	是	单位为分
        银联优惠信息	couponInfo	String(500)	否	银联通道且有优惠活动时出现，字段说明详见2.6优惠信息字段说明
        订单原始金额	orderOrigAmt	String(13)	否	单位为分，请求上送orderOrigAmt则对应返回该内容
        订单优惠金额	orderCouponAmt	String(13)	否	单位为分，请求上送orderCouponAmt则对应返回该内容
        优惠券信息	promotionDetail	String	否	微信、支付宝优惠券信息，规则见2.6优惠信息字段说明
        交易币种	currencyCode	String(3)	是	目前只支持人民币（156）
        支付方式	payType	String(2)	是	目前支持：支付宝/微信/银联
        ZF：支付宝
        WX：微信
        YL：银联
        用户标识	openId	String(128)	否	支付宝支付：用户在支付宝系统的唯一标识
        微信支付：商户上送sub_appid时返回用户在sub_appid下的sub_openid
        付款银行	payBank	String(32)	否	支付宝支付：
                                        ALIPAYACCOUNT 支付宝账户
                                        PCREDIT 蚂蚁花呗
                                        DEBIT_CARD 借记卡
                                        CREDIT_CARD 信用卡
                                        MIXED_CARD 借贷合一卡 等
        微信支付：CMB_CREDIT 等
        第三方订单号	thirdOrderId	String(64)	否	支付宝、微信侧的订单号
        支付宝账户	buyerLogonId	String(64)	否	支付宝支付的交易返回，如：koh***@sandbox.com
        订单发送时间	txnTime	String(14)	是	支付订单生成的时间，格式为yyyyMMddHHmmss
        订单完成日期	endDate	String(8)	否	第三方系统返回的交易完成日期，此日期仅在交易成功是返回，不保证是实际清算日期，格式为yyyyMMdd
        订单完成时间	endTime	String(6)	否	第三方系统返回的交易完成时间，此时间仅在交易成功是返回，格式为HHmmss
        支付宝和微信通道的成功交易返回此字段
        银联通道的交易无论成功与否，均不返回此字段
        商户保留域	mchReserved	String(128)	否	若原交易有上送此字段，则返回此字段

         */

        JSONObject result = new JSONObject();
        System.out.println(params);
        String requestBodyString = params.toJSONString();
        Map<String, String> respData = new HashMap<>();
        /*
        版本号	version	    String(5)	    是	固定为0.0.1
        编码方式	encoding	String(20)	    是	固定为UTF-8
        签名	    sign	    String(1024)	是
        签名方法	signMethod	String(2)	    是	02：SM2
        返回码	returnCode	String(7)	    是	SUCCESS/FAIL,SUCCESS表示商户接收通知成功并校验成功
                    以下字段仅在returnCode为SUCCESS时返回
        响应码	respCode	String(10)	    是	商户返回的业务错误码，成功为SUCCESS，失败为FAIL
        应答信息	respMsg	    String(256)	    否	请求处理失败的详细描述信息

        */


        //设置响应数据
        respData.put("version", "0.0.1");//版本号，固定为0.0.1(必传)
        respData.put("encoding", "UTF-8");//编码方式，固定为UTF-8(必传)
        respData.put("signMethod", "02");//签名方法，固定为02，国密
        try {
            respData.put("returnCode", "SUCCESS"); //SUCCESS表示商户接收通知成功并校验成功
            //非空校验
            if (requestBodyString == null || "".equals(requestBodyString.trim())) {
                respData.put("returnCode", "FAIL");
                return respData;
            }
            Map<String, String> requestBodyMap = str2Map(requestBodyString);
            Map<String, String> resultMap = requestBodyMap.entrySet().stream().collect(Collectors.toMap(e -> SignatureUtil.decode(e.getKey()), e -> SignatureUtil.decode(e.getValue())));
            if (resultMap == null) {
                respData.put("returnCode", "FAIL");
                return respData;
            }
            String sign = resultMap.remove("sign");
            //对待加签内容进行排序拼接
            String contentStr = SignatureUtil.getSignContent(resultMap);
            //验证签名-使用招行公钥进行验签
            boolean flag = SM2Util.sm2Check(contentStr,sign, publicKey);
            if (!flag) {
                //验签失败
                System.out.println("验签失败");
                respData.put("returnCode", "FAIL");
                return respData;
            }
            System.out.println("验签成功");
            //......（处理自身业务逻辑）
            //......
            //......
            respData.put("respCode", "SUCCESS");//业务错误码，成功为SUCCESS，失败为FAIL
            /*如果处理自身业务逻辑发生错误，返回
            respData.put("respCode","FAIL");
            respData.put("respMsg","error_msg");
             */
            //对待加签内容进行排序拼接
            String signContent = SignatureUtil.getSignContent(respData);
            //加签-使用商户私钥加签
            respData.put("sign", SM2Util.sm2Sign(signContent, privateKey));
            System.out.println("加签成功");
            return respData;
        } catch (Exception e) {
            e.printStackTrace();
            respData.put("returnCode", "FAIL");
            return respData;
        }
    }
    private Map<String, String> str2Map(String str) {
        Map<String, String> result = new HashMap<>();
        String[] results = str.split("&");
        if (results != null && results.length > 0) {
            for (int var = 0; var < results.length; ++var) {
                String pair = results[var];
                String[] kv = pair.split("=", 2);
                if (kv != null && kv.length == 2) {
                    result.put(kv[0], kv[1]);
                }
            }
        }
        return result;
    }
}
